(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     57112,       1358]*)
(*NotebookOutlinePosition[     58403,       1402]*)
(*  CellTagsIndexPosition[     58299,       1395]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["Self Organizing Maps Package", "Title"],

Cell["\<\
Written by Claudio Mattiussi
LIS - I2S - STI - EPFL\t
1015 Lausanne, Switzerland
email: claudio.mattiussi@epfl.ch\
\>", "Text"],

Cell[CellGroupData[{

Cell["Infos", "Subtitle"],

Cell[BoxData[
    \(\(\( (*\ \(:\)\(Title : \ 
          Self\ Organizing\ Maps\)\ *) \)\(\n\)\(\n\)\( (*\ \
\(:\)\(\(Context\)\(:\)\)\ *) \)\(\n\)\(\n\)\( (*\ \(:\)\(Author : \ 
          Claudio\ Mattiussi\ \n
        \t\t\tLSA2\  - \ I2S\  - \ STI\  - \ EPFL\n
        \t\t\t1015\ Lausanne\n
        \t\t\tSwitzerland\n
        \t\t\temail : \ 
          claudio . mattiussi@epfl . 
            ch\)\n*) \)\(\n\)\(\n\)\( (*\ \(:\)\(Summary : \ \n\t
          This\ package\ is\ intended\ as\ an\ educational\ tool\ for\ \n
        \texperimenting\ with\ self - 
          organizing\ \((Kohonen)\)\ \(\(maps\)\(.\)\)\)\n*) \)\(\n\)\(\n\)\( \
(*\ \(:\)\(Package\ \(Version : \ 
            0.1\)\)\ *) \)\(\n\)\(\n\)\( (*\ \(:\)\(Mathematica\ \(Version : \
\ Tested\ on\ v . \ 4.10\)\)\ *) \)\(\n\)\(\n\)\( (*\ \(\(:\)\(Copyright : \ 
          Copyright\ 2002\)\), \ 
      Claudio\ Mattiussi\ *) \)\(\n\)\(\n\)\( (*\ \(:\)\(History : \n\t\
\(March\ 2002 : \ 
            Version\ 0.1\)\)\n*) \)\(\n\)\(\n\)\( (*\ \(:\)\(Keywords : \ \n\t
          Self - organizing\ Maps\n
        \t
        Kohonen\ Maps\)\n*) \)\(\n\)\(\n\)\( (*\ \
\(:\)\(\(Limitations\)\(:\)\)\ *) \)\(\n\)\(\n\)\( (*\ \
\(:\)\(\(Discussion\)\(:\)\)\ *) \)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Definition Module", "Subtitle"],

Cell[CellGroupData[{

Cell["Usage", "Section"],

Cell[BoxData[
    \(\(\(\n\)\( (*\(BeginPackage["\<SelfOrganizingMaps`\>"];\)*) \)\(\n\
\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(dimInputSpace::usage\  = \n\t"\<dimInputSpace[] returns the \
dimension of the input space, i.e., the number of components of each input \
vector\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(numRowsOutputSpace::usage\  = \n\t"\<numRowsOutputSpace[] \
returns the numbers of rows of the output space, i.e., the number of rows of \
the Kohonen layer.\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(numColumnsOutputSpace::usage\  = \n\t\
"\<numColumnsOutputSpace[] returns the numbers of columns of the output \
space, i.e., the number of columns of the Kohonen layer.\>";\)\)\)], \
"InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(numPlanesOutputSpace::usage\  = \n\t"\<numPlanesOutputSpace[] \
returns the numbers of planes of the output space, i.e., the number of planes \
of the Kohonen layer.\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(generateSeparatorList::usage\  = \n\t\
"\<generateSeparatorList[numberOfSeparatorPoints] returns a list of integers \
in the range {1,numberOfSeparatorPoints} with increasingly wide gaps between \
adjacent values. This list is then used by the procedure partitionList to \
partition the list of test vectors in order to generate a limited number of \
intermediate results, with an emphasis on the initial phase of the unfolding \
process\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(generateUniformlyDistributedRandomTrainingPointList::usage\  \
= \n\t"\<generateUniformlyDistributedRandomTrainingPointList[\
numberOfTrainingPoints,inputRange] returns a list of training vectors suited \
to the output geometry specified using the function \
setGeometricParameters\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(generateUniformlyDistributedRandomWeightList::usage\  = \n\t"\
\<generateUniformlyDistributedRandomWeightList[inputRange] returns a list of \
weigths suited to the output geometry specified using the function \
setGeometricParameters\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(initializeAlgorithm::usage\  = \n\t"\<initializeAlgorithm[] \
presets all the internal variables for the execution of the algorithm. In \
particular, it clears the algorithm step counter\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(partitionList::usage\  = \n\t\
"\<partitionList[trainingPointList,separatorList] partitions the \
trainingPointList according to the criterion given by separatorList. This \
allows the generation of a limited number of intermediate results during the \
execution of the algorithm, with an emphasis on the initial phase of the \
unfolding process\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(setDistanceFunction::usage\  = \n\t"\<setDistanceFunction[] \
assigns the default distance function, i.e., the function used within the \
neighborhood to determine the distance between output neurons. The default \
distance function is part of the Options of setDistanceFunction. The form \
setDistanceFunction[distanceFunction->Compile[{i,j,k,i0,j0,k0},d[i,j,k,i0,j0,\
k0]] is used to assign (as a compiled function) the function \
d[i,j,k,i0,j0,k0] as distance function, where i,j,k,i0,j0,k0 are the indexes \
of the two neurons whose distance is to be determined\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(setGeometricParameters::usage\  = \n\t\
"\<setGeometricParameters[dimInputSpace\[Rule]a,numRowsOutputSpace\[Rule]b,\
numColumnsOutputSpace\[Rule]c,numPlanesOutputSpace\[Rule]d] assigns the \
geometric parameters of the input and output spaces. The parameters must be \
positive integers\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(setLearningRateFunction::usage\  = \n\t\
"\<setLearningRateFunction[] assigns the default learning rate function, \
i.e., the function used within the algorithm as a (usually decreasing with \
time) multiplicative coefficient of the weight update term. The default \
learning rate function is part of the Options of setLearningRateFunction. The \
form setLearningRateFunction[learningRateFunction->Compile[n,lrf[n]] is used \
to assign (as a compiled function) the function lrf[n] as learning rate \
function, where n is the algorithm step counter\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(setMatchFunction::usage\  = \n\t"\<setMatchFunction[] assigns \
the default matching function, i.e., the function used to select the winning \
output neuron. The default matching function is part of the Options of \
setMatchFunction. The form \
setMatchFunction[matchFunction->Compile[{x,w},f[x,w]] is used to assign (as a \
compiled function) the function f[x,w] as matching function, where x is the \
input test vector and w is the weight vector\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(setNeighborhoodFunction::usage\  = \n\t\
"\<setNeighborhoodFunction[] assigns the default neighborhood function, i.e., \
the function used within the algorithm as a multiplicative coefficient of the \
weight update term. The default neighborhood function is part of the Options \
of setNeighborhoodFunction. The form setNeighborhoodFunction[		\
neighborhoodFunction->Compile[{i,j,k,i0,j0,k0,n},nf[i,j,k,i0,j0,k0,n]] is \
used to assign (as a compiled function) the function nf[{i,j,k,i0,j0,k0,n] as \
neighborhood function, where i0,j0,k0 are the indexes of the winning neuron, \
i,j,k, are the indexes of the neuron whose weight are being updated, and n is \
the algorithm step counter\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(sequenceOfUpdatesWeightList::usage\  = \n\t\
"\<sequenceOfUpdatesWeightList[weightList, partitionedTrainingPointList] \
returns a sequence of weight sets, obtained by subjecting weightList to the \
training represented by partitionedTrainingPointList. If \
partitionedTrainingPointList is partitioned, only the results corresponding \
to the application last element of each sublist is returned.\>";\)\)\)], \
"InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(showHistory::usage\  = \n\t"\<showHistory[historyOfWeights, \
separatorList, trainingPointList] displays the sequence of weight plots \
represented by historyOfWeights, assuming that the algorithm has been run \
with the training points trainingPointList partitioned by \
separatorList.\>";\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(showTrainingPoints::usage\  = \n\t\
"\<showTrainingPoints[trainingPointList, options] plots the set of training \
points trainingPointList\>";\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Options", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          setGeometricParameters] = \ \n\t\t{\n\t\tdimInputSpace \[Rule] 
            2, \n\t\tnumRowsOutputSpace \[Rule] 
            4, \n\t\tnumColumnsOutputSpace \[Rule] 
            5, \n\t\tnumPlanesOutputSpace \[Rule] 
            2, \n\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          setLearningRateFunction] = \n\t\t{\n\t\teta0 \[Rule] 
            0.1, \n\t\ttau2 \[Rule] 
            1000. , \n\t\tlearningRateFunction -> \n\t\t\tCompile[\n\t\t\t\t\
n, \n\t\t\t\teta0\ \ Exp[\(-n\)/tau2]\n\t\t\t\t]\n\t\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          setDistanceFunction] = \n\t\t{\n\t\tdistanceFunction -> \n\t\t\t\
Compile[\n\t\t\t\t{i, j, k, i0, j0, 
                k0}, \n\t\t\t\tSqrt[\((\((i - i0)\)^2 + \((j - 
                          j0)\)^2 + \((k - 
                          k0)\)^2)\)]\n\t\t\t\t]\n\t\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          setNeighborhoodFunction] = \n\t\t{\n\t\tsigma0 -> 
            Sqrt[\n\t\t\t\t\t\((\((numRowsOutputSpace[] - 1)\)/
                      2)\)^2 + \n\t\t\t\t\t\((\((numColumnsOutputSpace[] - 
                          1)\)/2)\)^2 + \n\t\t\t\t\t\
\((\((numPlanesOutputSpace[] - 1)\)/2)\)^2\n\t\t\t\t\t], \n\t\ttau1 \[Rule] 
            1000. , \n\t\tneighborhoodFunction -> \n\t\t\tCompile[\n\t\t\t\t\
{i, j, k, i0, j0, k0, 
                n}, \n\t\t\t\t\ Exp[\(-\((\(distanceFunction[]\)[i, j, k, i0, 
                          j0, k0])\)^2\)/\n\t\t\t\t\ \((2\ \((sigma0\ Exp\ \
[\(-n\)/tau1])\)^2)\)]\n\t\t\t\t]\n\t\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          setMatchFunction] = \n\t\t{\n\t\tmatchFunction -> \n\t\t\t\
Compile[{x, w}, \((x - w)\) \((x - w)\)]\n\t\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          showHistory] = \n\t\t{\n\t\tAspectRatio \[Rule] 
            Automatic, \n\t\tPlotRange \[Rule] {{0, 1}, {0, 1}, {0, 
                1}}, \n\t\tBoxed \[Rule] True, \n\t\tLighting \[Rule] 
            False, \n\t\tcuboidSideLength \[Rule] 
            0.05, \n\t\tshowWinner \[Rule] 
            True, \n\t\ttrainingPointSize \[Rule] 
            0.02, \n\t\tshowActiveTrainingPoint \[Rule] 
            True, \n\t\tshowAllTrainingPoints \[Rule] 
            False, \[IndentingNewLine]\t
          showOnlyFinalConfiguration \[Rule] False, \n\t\tshowLines \[Rule] 
            False, \n\t\tlineThickness \[Rule] 
            0.02, \n\t\tViewPoint \[Rule] {\(-2.289\), \ \(-2.439\), \ 
              0.514}\n\t\t};\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Options[
          showTrainingPoints] = \n\t\t{\n\t\ttrainingPointSize \[Rule] 
            0.02, \n\t\tPlotRange \[Rule] {{0, 1}, {0, 1}, {0, 
                1}}, \n\t\tPlotLabel -> "\<\>"\n\t\t};\)\)\)], "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Implementation Module", "Subtitle"],

Cell[CellGroupData[{

Cell["Open Private Context", "Section"],

Cell[BoxData[
    \( (*\nBegin["\<`Private`\>"]; \nUnprotect[];\n*) \)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Geometrical Parameters", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[setGeometricParameters];\)\n
    \t\(setGeometricParameters\ [\n\t\toptions___Rule\n\t\t]\  := \n\t\t\
Module[\n\t\t\t{\n\t\t\t}, \n\t\t\tdimInptSpc\  = \ 
            Max[\n\t\t\t\t\t\t\(dimInputSpace /. \n\t\t\t\t\t\t{options}\) \
/. \n\t\t\t\t\t\tOptions[
                  setGeometricParameters], \n\t\t\t\t\t\t1]; \n\t\t\t\
rowsOutSpc\  = \ 
            Max[\n\t\t\t\t\t\t\(numRowsOutputSpace /. \n\t\t\t\t\t\t{options}\
\) /. \n\t\t\t\t\t\tOptions[
                  setGeometricParameters], \n\t\t\t\t\t\t1]; \n\t\t\t\
colsOutSpc\  = \ 
            Max[\n\t\t\t\t\t\t\(numColumnsOutputSpace /. \n\t\t\t\t\t\t\
{options}\) /. \n\t\t\t\t\t\tOptions[
                  setGeometricParameters], \n\t\t\t\t\t\t1]; \n\t\t\t\
plnsOutSpc\  = \ 
            Max[\n\t\t\t\t\t\t\(numPlanesOutputSpace /. \n\t\t\t\t\t\t\
{options}\) /. \n\t\t\t\t\t\tOptions[
                  setGeometricParameters], \n\t\t\t\t\t\t1]; \n\t\t\t\
ColumnForm[{\n\t\t\t\t\ \ "\<dimInputSpace->\>" <> 
                ToString[
                  dimInptSpc], \n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \
"\<numRowsOutputSpace->\>" <> 
                ToString[
                  rowsOutSpc], \n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \
"\<numColumnsOutputSpace->\>" <> 
                ToString[
                  colsOutSpc], \n\t\t\t\ \ \ \ \ \ \ \ \ \ \ \
"\<numPlanesOutputSpace->\>" <> 
                ToString[
                  plnsOutSpc]\n\t\t\t\ \ \ \ \ \ \ \ \ \ \ }]\n\t\t\t];\)\)\(\
\t\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(dimInputSpace[] := dimInptSpc;\)\n
    \t\(numRowsOutputSpace[] := rowsOutSpc;\)\n
    \t\(numColumnsOutputSpace[] := colsOutSpc;\)\n
    \t\(numPlanesOutputSpace[] := plnsOutSpc;\)\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Learning Rate Function", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[setLearningRateFunction];\)\n
    \t\(setLearningRateFunction[\n\t\toptions___Rule\n\t\t] := \n\t\t\
Module[{}, \n\t\t\tlrnRateFunct = \n\t\t\t\t\(\((\(learningRateFunction /. \n\
\t\t\t\t\t{options}\) /. \n\t\t\t\t\tOptions[
                      setLearningRateFunction]\n\t\t\t\t)\) /. \n\t\t\t\t\
{options}\) /. \n\t\t\t\tOptions[
                setLearningRateFunction]; \n\t\t\tglobalEta0 = \(eta0 /. \
{options}\) /. 
              Options[setLearningRateFunction];\n\t\t\t];\)\)\)\)], \
"InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(learningRateFunction[] := lrnRateFunct;\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Distance Function", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[setDistanceFunction];\)\n
    \t\(setDistanceFunction[\n\t\toptions___Rule\n\t\t] := \n\t\tModule[{}, \n\
\t\t\t\(dstFunct = \n\t\t\t\t\(distanceFunction /. \n\t\t\t\t\t{options}\) \
/. \n\t\t\t\t\tOptions[setDistanceFunction];\)\n\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(distanceFunction[] := dstFunct;\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Neighborhood Function", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[setNeighborhoodFunction];\)\n
    \t\(setNeighborhoodFunction[\n\t\toptions___Rule\n\t\t] := \n\t\t\
Module[{}, \n\t\t\t\(ngbhdFunct = \n\t\t\t\t\(\((\(neighborhoodFunction /. \n\
\t\t\t\t\t{options}\) /. \n\t\t\t\t\tOptions[
                        setNeighborhoodFunction]\n\t\t\t\t)\) /. \n\t\t\t\t\
{options}\) /. \n\t\t\t\tOptions[
                  setNeighborhoodFunction];\)\n\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(neighborhoodFunction[] := ngbhdFunct;\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Match Function", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[setMatchFunction];\)\n
    \t\(setMatchFunction[\n\t\toptions___Rule\n\t\t] := \n\t\tModule[{}, \n\t\
\t\tmtcFunct = \n\t\t\t\t\(matchFunction /. \n\t\t\t\t\t{options}\) /. \n\t\t\
\t\t\tOptions[setMatchFunction]; \n\t\t\tSetAttributes[mtcFunct, 
            Listable];\n\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(matchFunction[] := mtcFunct;\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Initialization Functions", "Section"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[initializeAlgorithm];\)\n
    \t\(initializeAlgorithm[\n\t\t] := \n\t\tModule[\n\t\t\t{\n\t\t\t}, \n\t\t\
\t\(algorithmStepCounter = 0;\)\n\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[generateUniformlyDistributedRandomWeightList];\)\n
    \t\(generateUniformlyDistributedRandomWeightList[\n\t\t\t{\n\t\t\t\
xRange_List, \n\t\t\tyRange_List: {0, 0}, \n\t\t\tzRange_List: {0, 
                0}, \n\t\t\thyperRange___List\n\t\t\t}\n\t\t] := \n\t\t\
Module[\n\t\t\t{\n\t\t\twghtList\n\t\t\t}, \n\t\t\twghtList = \n\t\t\t\t\
Table[\n\t\t\t\t\tFlatten[\n\t\t\t\t\t{\n\t\t\t\t\tRandom[Real, 
                    xRange], \n\t\t\t\t\tRandom[Real, 
                    yRange], \n\t\t\t\t\tRandom[Real, 
                    zRange], \n\t\t\t\t\tMap[Random[Real, #] &, 
                    Evaluate[
                      Join[{hyperRange}, {}]]]\n\t\t\t\t\t}\n\t\t\t\t\t], \n\t\
\t\t\t\t{i, 1, rowsOutSpc}, \n\t\t\t\t\t{j, 1, colsOutSpc}, \n\t\t\t\t\t{k, 
                1, plnsOutSpc}\n\t\t\t\t\t]; \n\t\t\twghtList\n\t\t\t];\)\)\)\
\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[
        generateUniformlyDistributedRandomTrainingPointList];\)\n
    \t\(generateUniformlyDistributedRandomTrainingPointList[\n\t\t\
nTrainingPoints_Integer?
            Positive, \n\t\t{\n\t\txRange_List, \n\t\tyRange_List: {0, 
                0}, \n\t\tzRange_List: {0, 
                0}, \n\t\thyperRange___List\n\t\t}\n\t\t] := \n\t\tModule[\n\t\
\t\t{\n\t\t\tsmplList, \n\t\t\ti, 
            j\n\t\t\t}, \n\t\t\tsmplList = \n\t\t\t\tTable[\n\t\t\t\t\t\
Flatten[\n\t\t\t\t\t{\n\t\t\t\t\tRandom[Real, xRange], \n\t\t\t\t\tRandom[
                    Real, yRange], \n\t\t\t\t\tRandom[Real, 
                    zRange], \n\t\t\t\t\tMap[Random[Real, #] &, 
                    Evaluate[
                      Join[{hyperRange}, {}]]]\n\t\t\t\t\t}\n\t\t\t\t\t], \n\t\
\t\t\t\t{j, 1, 
                nTrainingPoints}\n\t\t\t\t\t]; \n\t\t\tsmplList\n\t\t\t];\)\)\
\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[generateSeparatorList];\)\n
    \t\(generateSeparatorList[\n\t\tn_, \n\t\tbase_:  2, \n\t\tthreshold_:  
            20\n\t\t] := \t\t\n\t\t\tSelect[\n\t\t\t\tUnion[\n\t\t\t\t\tSort[\
\n\t\t\t\t\t\tFlatten[\n\t\t\t\t\t\t\tTable[\n\t\t\t\t\t\t\t\ti*
                      base^j, \n\t\t\t\t\t\t\t\t{i, 1, 
                      threshold}, \n\t\t\t\t\t\t\t\t{j, 0, 
                      Log[base, 
                        Max[n/threshold, 
                          n]]}\n\t\t\t\t\t\t\t\t]\n\t\t\t\t\t\t\t]\n\t\t\t\t\t\
\t]\n\t\t\t\t\t], \n\t\t\t\t\t# <= n &\n\t\t\t\t] /; \n\t\t\t\tbase > 
              1\  && \n\t\t\t\tthreshold > 1;\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(partitionList[list_List, 
        separatorList_List] := \n\t\tWith[\n\t\t\t{\n\t\t\tdimList = 
            Length[list], \n\t\t\tdimSepList = 
            Length[separatorList]\t\t\n\t\t\t}, \n\t\t\tTable[\n\t\t\t\tIf[\n\
\t\t\t\t\tseparatorList[\([i]\)] <= 
                separatorList[\([i + 
                      1]\)]\  && \n\t\t\t\t\tseparatorList[\([i + 1]\)] <= 
                dimList, \n\t\t\t\t\tTake[\n\t\t\t\t\t\tlist, \n\t\t\t\t\t\t\
{separatorList[\([i]\)], 
                separatorList[\([i + 1]\)] - 
                  1}\n\t\t\t\t\t\t]\n\t\t\t\t\t], \n\t\t\t\t{i, 
            1, \((dimSepList - 1)\)}\n\t\t\t\t]\n\t\t\t]\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Update Functions", "Section"],

Cell[CellGroupData[{

Cell["Single step", "Subsubsection"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[updateWeightList, 
        listOfWinners];\)\[IndentingNewLine]
    \t\(listOfWinners = {{0, 0, 0}};\)\[IndentingNewLine]
    \tupdateWeightList[weightList_List, 
        trainingPointVector_List] := \[IndentingNewLine]\t\tModule[{iWn, jWn, 
          kWn, matchMatrix, neighborhoodMatrix, matchLevelList, 
          newWeightList, 
          learningRate}, \[IndentingNewLine]\t\t{iWn, jWn, 
            kWn} = \[IndentingNewLine]\t\tFirst[
            Position[
              matchMatrix = Inner[mtcFunct, weightList, trainingPointVector], 
              Min[matchMatrix]]]; \[IndentingNewLine]\t\tAppendTo[
          listOfWinners, {iWn, jWn, 
            kWn}]; \[IndentingNewLine]\t\tneighborhoodMatrix = \
\[IndentingNewLine]\t\tTable[\(neighborhoodFunction[]\)[i, j, k, iWn, jWn, 
              kWn, algorithmStepCounter], \[IndentingNewLine]\t\t{i, 1, 
              rowsOutSpc}, \[IndentingNewLine]\t\t{j, 1, 
              colsOutSpc}, \[IndentingNewLine]\t\t{k, 1, 
              plnsOutSpc}]; \[IndentingNewLine]\t\tlearningRate = \
\(learningRateFunction[]\)[
            algorithmStepCounter]; \[IndentingNewLine]\t\tnewWeightList = 
          weightList + 
            learningRate*neighborhoodMatrix*
              Map[trainingPointVector - # &, 
                weightList, {3}]; \[IndentingNewLine]\t\t\
\(algorithmStepCounter++\); \[IndentingNewLine]\t\tnewWeightList\
\[IndentingNewLine]\t\t]\)\)\)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Multiple steps", "Subsubsection"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[sequenceOfUpdatesWeightList];\)\[IndentingNewLine]
    \t\(sequenceOfUpdatesWeightList[\[IndentingNewLine]\t
          weightList_List, \[IndentingNewLine]\t
          possiblyPartitionedTrainingPointVectorList_List\[IndentingNewLine]\t\
] := \[IndentingNewLine]\t
        FoldList[\[IndentingNewLine]\t\tFold[
              updateWeightList, #1, #2] &, \[IndentingNewLine]\t\tweightList, \
\[IndentingNewLine]\t\tpossiblyPartitionedTrainingPointVectorList\
\[IndentingNewLine]\t\t];\)\)\)\)], "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Graphical Functions", "Section"],

Cell[BoxData[
    \(\(Needs["\<Graphics`Polyhedra`\>"];\)\)], "InputOnly"],

Cell[CellGroupData[{

Cell["Notebook", "Subsubsection"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[showTrainingPoints];\)\n
    \t\(showTrainingPoints[\n\t\ttrngPnttList_List, \n\t\toptions___Rule\n\t\t\
] := \n\t\tModule[\n\t\t\t{\n\t\t\tpltRng = \((\(PlotRange /. {options}\) /. 
                  Options[
                    showTrainingPoints])\), \n\t\t\tsmallPltRng\t\t\t\n\t\t\t\
}, \n\t\t\tpltRng = 
            Take[pltRng, 
              Min[3, Length[
                  pltRng]]]; \n\t\t\tsmallPltRng = {\(\(-trainingPointSize\) /. \
{options}\) /. Options[
                  showTrainingPoints], \n\t\t\t\t\(trainingPointSize /. \
{options}\) /. Options[showTrainingPoints]}; \t\t\n\t\t\tIf[
            dimInputSpace[] \[Equal] 2, \n\t\t\t\tpltRng = 
              AppendTo[pltRng, smallPltRng]\n\t\t\t]; \n\t\t\tIf[
            dimInputSpace[] \[Equal] 1, \n\t\t\t\tpltRng = 
              AppendTo[pltRng, smallPltRng]; \n\t\t\t\tpltRng = 
              AppendTo[pltRng, 
                smallPltRng]\n\t\t\t]; \n\t\t\tShow[\n\t\t\t\tGraphics3D[\n\t\
\t\t\t\t{\n\t\t\t\t\tRGBColor[0.2, 0.2, 
                  0.2], \n\t\t\t\t\tPointSize[\(trainingPointSize /. \
{options}\) /. Options[
                      showTrainingPoints]], \n\t\t\t\t\tTable[\n\t\t\t\t\t\t\
Point[Take[trngPnttList[\([i]\)], 
                      Min[3, Length[
                          trngPnttList[\([i]\)]]]]], \n\t\t\t\t\t\t{i, 1, 
                    Length[trngPnttList]}\t\n\t\t\t\t\t\t]\n\t\t\t\t\t\t}\n\t\
\t\t\t\t], \t\n\t\t\t\tAspectRatio \[Rule] \((\(AspectRatio /. {options}\) /. 
                  Options[showHistory])\), \n\t\t\t\tPlotRange \[Rule] 
              pltRng, \n\t\t\t\tBoxed \[Rule] \((\(Boxed /. {options}\) /. 
                  Options[
                    showHistory])\), \n\t\t\t\tLighting \[Rule] \((\(Lighting \
/. {options}\) /. Options[
                    showHistory])\), \n\t\t\t\tPlotLabel -> \((\(PlotLabel /. \
{options}\) /. Options[
                    showHistory])\), \[IndentingNewLine]\t\t\tViewPoint -> \
\((\(ViewPoint /. {options}\) /. 
                  Options[
                    showHistory])\), \n\t\t\t\tDisplayFunction \[Rule] \
\((\(DisplayFunction /. {options}\) /. 
                  DisplayFunction \[Rule] $DisplayFunction)\)\n\t\t\t\t]\n\t\t\
\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[generateCuboidsMesh3D];\)\n
    \t\(generateCuboidsMesh3D[\n\t\twL_List, \n\t\tsprtrs_List, \n\t\t\
trngPnt_List, \n\t\tstep_Integer, \n\t\toptions___Rule\n\t\t] := \n\t\t\
Module[\n\t\t\t{\n\t\t\tcuboidHalfSize = 
              1/2\ \((\(cuboidSideLength /. {options}\) /. 
                    Options[
                      showHistory])\)\[IndentingNewLine]\t\t\t}, \n\t\t\t\
Graphics3D[\n\t\t\t\t{\n\t\t\t\tIf[\(showLines /. {options}\) /. 
                  Options[
                    showHistory], \n\t\t\t\t\t{\n\t\t\t\t\t\
Thickness[\(lineThickness /. {options}\) /. 
                      Options[showHistory]], \n\t\t\t\t\tMap[Line, 
                    Map[Take[#, Min[3, Length[#]]] &, 
                      Transpose[
                        wL[\([step]\)], {1, 2, 
                          3}], {3}], {2}], \n\t\t\t\t\tMap[Line, 
                    Map[Take[#, Min[3, Length[#]]] &, 
                      Transpose[
                        wL[\([step]\)], {1, 3, 
                          2}], {3}], {2}], \n\t\t\t\t\tMap[Line, 
                    Map[Take[#, Min[3, Length[#]]] &, 
                      Transpose[
                        wL[\([step]\)], {3, 2, 
                          1}], {3}], {2}]\n\t\t\t\t\t}, \n\t\t\t\t\t{}\t\t\n\t\
\t\t\t\t], \n\t\t\t\tIf[\(\((step > 1)\) && \((step < 
                          Length[
                            sprtrs])\) && \n\t\t\t\t\tshowActiveTrainingPoint \
/. {options}\) /. Options[showHistory], \n\t\t\t\t\t{\n\t\t\t\t\tRGBColor[
                    0.2, 0.2, 0.2], \n\t\t\t\t\tApply[\n\t\t\t\t\t\t\tList, 
                    Polyhedron[
                      Dodecahedron, \[IndentingNewLine]\t\t\t\t\tTake[
                        trngPnt[\([sprtrs[\([step]\)] - 
                              1]\)], \[IndentingNewLine]\t\t\t\t\t\tMin[3, 
                          Length[
                            trngPnt[\([sprtrs[\([step]\)] - 
                                  1]\)]]]], \[IndentingNewLine]\t\t\t\t\t\t2  
                        cuboidHalfSize]\n\t\t\t\t\t\t\t]\n\t\t\t\t\t}, \n\t\t\
\t\t\t{\n\t\t\t\t\tPointSize[
                    0.0001], \n\t\t\t\t\tPoint[{0, 0, 
                      0}]\n\t\t\t\t\t}\n\t\t\t\t], \n\t\t\t\tTable[\n\t\t\t\t\
\t{\n\t\t\t\t\tIf[\n\t\t\t\t\t\tnumPlanesOutputSpace[] \[Equal] 
                        1 || \n\t\t\t\t\t\tnumRowsOutputSpace[] \[Equal] 
                        1 || \n\t\t\t\t\t\tnumColumnsOutputSpace[] \[Equal] 
                        1, \n\t\t\t\t\t\tHue[\n\t\t\t\t\t\t\t\((k + i + 
                            j)\)/\n\t\t\t\t\t\t\t\((numPlanesOutputSpace[] + 
                            numRowsOutputSpace[] + 
                            numColumnsOutputSpace[])\)\n\t\t\t\t\t\t\t], \n\t\
\t\t\t\t\tRGBColor[\n\t\t\t\t\t\t\tk/
                        numPlanesOutputSpace[], \n\t\t\t\t\t\t\ti/
                        numRowsOutputSpace[], \n\t\t\t\t\t\t\tj/
                        numColumnsOutputSpace[]\n\t\t\t\t\t\t\t]\n\t\t\t\t\t\t\
], \n\t\t\t\t\tIf[\n\t\t\t\t\t\t\(\((step > 1)\)\  && \((step < 
                              Length[sprtrs])\) && \n\t\t\t\t\t\t{i, j, 
                              k} \[Equal] 
                            listOfWinners[\([\ 
                                sprtrs[\([step]\)]\ ]\)] && \n\t\t\t\t\t\t\
showWinner /. {options}\) /. 
                      Options[
                        showHistory], \t\t\t\t\t\t\n\t\t\t\t\t\tApply[\n\t\t\t\
\t\t\t\tList, Polyhedron[
                        GreatStellatedDodecahedron, \[IndentingNewLine]\t\t\t\
\t\tTake[\(wL[\([step]\)]\)[\([i, j, k]\)], 
                          Min[3, 
                            Length[\(wL[\([step]\)]\)[\([i, j, 
                                  k]\)]]]], \[IndentingNewLine]\t\t\t\t\t\
cuboidHalfSize]\n\t\t\t\t\t\t\t], \t\t\t\t\t\t\t\n\t\t\t\t\t\tCuboid[
                      Take[\(wL[\([step]\)]\)[\([i, j, k]\)], 
                          Min[3, 
                            Length[\(wL[\([step]\)]\)[\([i, j, k]\)]]]] - 
                        cuboidHalfSize, \[IndentingNewLine]\t\t\t\t\t\
Take[\(wL[\([step]\)]\)[\([i, j, k]\)], 
                          Min[3, 
                            Length[\(wL[\([step]\)]\)[\([i, j, k]\)]]]] + 
                        cuboidHalfSize]\n\t\t\t\t\t\t]\n\t\t\t\t\t}, \n\t\t\t\
\t\t{i, 1, numRowsOutputSpace[]}, \n\t\t\t\t\t{j, 1, 
                  numColumnsOutputSpace[]}, \n\t\t\t\t\t{k, 1, 
                  numPlanesOutputSpace[]}\n\t\t\t\t\t]\n\t\t\t\t}\n\t\t\t\t]\n\
\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[showHistory];\)\n
    \t\(showHistory[\n\t\thstryOfWghts_List, \n\t\tsprtrLst_List, \n\t\t\
trngPntLst_List, \n\t\toptions___Rule\n\t\t] := \n\t\tModule[\n\t\t\t{\n\t\t\t\
pltRng = \((\(PlotRange /. {options}\) /. 
                  Options[
                    showHistory])\), \n\t\t\tsmallPltRng, \[IndentingNewLine]\
\t\twghtsToProcess\t\t\t\n\t\t\t}, \[IndentingNewLine]\t\twghtsToProcess = 
            If[\[IndentingNewLine]\t\t\(showOnlyFinalConfiguration /. \
{options}\) /. Options[showHistory], \[IndentingNewLine]\t\t{Last[
                  hstryOfWghts]}, \[IndentingNewLine]\t\thstryOfWghts\
\[IndentingNewLine]\t\t]; \n\t\t\tpltRng = 
            Take[pltRng, 
              Min[3, Length[
                  pltRng]]]; \n\t\t\tsmallPltRng = {\(-\((\(cuboidSideLength /. \
{options}\) /. Options[
                      showHistory])\)\), \n\t\t\t\t\ \((\(cuboidSideLength /. \
{options}\) /. Options[showHistory])\)}; \t\t\n\t\t\tIf[
            dimInputSpace[] \[Equal] 2, \n\t\t\t\tpltRng = 
              AppendTo[pltRng, smallPltRng]\n\t\t\t]; \n\t\t\tIf[
            dimInputSpace[] \[Equal] 1, \n\t\t\t\tpltRng = 
              AppendTo[pltRng, smallPltRng]; \n\t\t\t\tpltRng = 
              AppendTo[pltRng, 
                smallPltRng]\n\t\t\t]; \n\t\t\tTable[\n\t\t\t\tShow[{\n\t\t\t\
\t\tIf[\(showAllTrainingPoints /. {options}\) /. 
                    Options[
                      showHistory], \n\t\t\t\t\t\tshowTrainingPoints[\n\t\t\t\
\t\t\t\ttrngPntLst, \n\t\t\t\t\t\t\ttrainingPointSize \[Rule] \
\((\(trainingPointSize /. {options}\) /. 
                          Options[
                            showTrainingPoints])\), \n\t\t\t\t\t\t\tPlotRange \
-> pltRng, \n\t\t\t\t\t\t\tDisplayFunction \[Rule] 
                      Identity\n\t\t\t\t\t\t], \n\t\t\t\t\t\t{}\n\t\t\t\t\t], \
\n\t\t\t\t\tgenerateCuboidsMesh3D[\n\t\t\t\t\t\twghtsToProcess, \n\t\t\t\t\t\t\
sprtrLst, \n\t\t\t\t\t\ttrngPntLst, \n\t\t\t\t\t\ti, \n\t\t\t\t\t\toptions\n\t\
\t\t\t\t]}, \t\n\t\t\t\t\tAspectRatio \[Rule] \((\(AspectRatio /. {options}\) \
/. Options[showHistory])\), \n\t\t\t\t\tPlotRange \[Rule] 
                pltRng, \n\t\t\t\t\tBoxed \[Rule] \((\(Boxed /. {options}\) /. 
                    Options[
                      showHistory])\), \n\t\t\t\t\tLighting \[Rule] \
\((\(Lighting /. {options}\) /. 
                    Options[
                      showHistory])\), \n\t\t\t\t\tPlotLabel -> "\<step = \>" \
<> \[IndentingNewLine]\t\t\tToString[\[IndentingNewLine]\t\t\t\t\
separatorList[\([\[IndentingNewLine]\t\t\t\t\tIf[\(showOnlyFinalConfiguration \
/. {options}\) /. Options[showHistory], \[IndentingNewLine]\t\t\t\t\t\tLength[
                            hstryOfWghts], \[IndentingNewLine]\t\t\t\t\t\ti\
\[IndentingNewLine]\t\t\t\t\t\t]\[IndentingNewLine]\t\t\t\t\t]\)]\
\[IndentingNewLine]\t\t\t\t], \n\t\t\t\t\tViewPoint -> \((\(ViewPoint /. \
{options}\) /. Options[
                      showHistory])\), \n\t\t\t\t\tDisplayFunction \[Rule] \
$DisplayFunction\n\t\t\t\t\t], \n\t\t\t\t{i, 1, 
              Length[wghtsToProcess]}\t\n\t\t\t\t]\n\t\t\t];\)\)\)\)], \
"InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(\(Clear[animateHistory];\)\n
    \t\(animateHistory[\n\t\thstryOfWghts_List, \n\t\tseparatorList_List, \n\t\
\ttrainingPointList_List, \n\t\toptions___Rule\n\t\t] := \n\t\tModule[\n\t\t\t\
{\n\t\t\t}, \n\t\t\tshowHistory[\n\t\t\t\thstryOfWghts, \n\t\t\t\t\
separatorList\n\t\t\t\ttrainingPointList, \n\t\t\t\toptions\n\t\t\t\t]\t\n\t\t\
\tSelectionMove[EvaluationNotebook[], All, 
              GeneratedCell]; \n\t\t\tSelectionMove[EvaluationNotebook[], 
            All, CellGroup]; \n\t\t\tFrontEndExecute[
            FrontEndToken["\<OpenCloseGroup\>"]]; \n\t\t\tSelectionAnimate[\n\
\t\t\t\tEvaluationNotebook[], \n\t\t\t\tAnimationDisplayTime \[Rule] 
              0.5, \n\t\t\t\tAnimationDirection \[Rule] 
              Forward\n\t\t\t\t]\n\t\t\t];\)\)\)\)], "InputOnly"],

Cell[BoxData[
    \(\(\(\t\)\(Animate[\n\t\t] := \n\t\tModule[\n\t\t{\n\t\t}, \n\t\t\
SelectionMove[EvaluationNotebook[], All, GeneratedCell]; \n\t\tSelectionMove[
            EvaluationNotebook[], All, CellGroup]; \n\t\tFrontEndExecute[
            FrontEndToken["\<OpenCloseGroup\>"]]; \n\t\tSelectionAnimate[\n\t\
\t\tEvaluationNotebook[], \n\t\t\tAnimationDisplayTime \[Rule] 
              0.5, \n\t\t\tAnimationDirection \[Rule] 
              Forward\n\t\t\t]\n\t\t];\)\)\)], "InputOnly"]
}, Open  ]],

Cell[CellGroupData[{

Cell["GraphicsLive3D", "Subsubsection"],

Cell[BoxData[
    RowBox[{"\t", 
      RowBox[{
        StyleBox[\(Live3DForm[graphics_, \ 
                animOpts___]\  := \[IndentingNewLine]\t
              Module[\n\t\t{\n\t\tgraphics3d\  = \ 
                    Switch[\n\t\t\tgraphics, \n\t\t\t\ _ContourGraphics, \n\t\
\t\t\ \ Graphics3D[SurfaceGraphics[
                          graphics]\ ], \n\t\t\t_DensityGraphics, \ \n\t\t\t\
Graphics3D[SurfaceGraphics[graphics]], \n\t\t\t_SurfaceGraphics, \ 
                      Graphics3D[
                        graphics], \ \n\t\t\t\ _, \ \n\t\t\tgraphics\n\t\t\t]\
\n\t\t}, \ \n\t\tSwitch[\n\t\t\tgraphics3d, \ \n\t\t\t_List, \ \n\t\t\t\
\(HoldForm[ShowAnimation]\)[\n\t\t\t\tNumberForm[\n\t\t\t\t\tInputForm[
                        N[graphics3d]], \ 5], \ \n\t\t\t\t\tInputForm[
                      N[Flatten[{animOpts}]]]\n\t\t\t\t\t], \ \n\t\t\t\t_, \ \
\n\t\t\t\tNumberForm[
                    InputForm[
                      N[graphics3d]], \ \n\t\t\t\t5\n\t\t\t\t]\n\t\t\t]\n\t\t\
];\),
          FormatType->StandardForm], 
        StyleBox["\n",
          FormatType->StandardForm], "\[IndentingNewLine]", 
        "\[IndentingNewLine]", 
        "\t", \(WriteLive3DForm[\n\t\tfilename_, \n\t\tgraphics_, \n\t\t\
animOpts___\n\t\t] := \n\t\tModule[\n\t\t\t{\n\t\t\tps, \n\t\t\tx\n\t\t\t}, \n\
\t\t\tps = Unprotect[
                  Real]; \n\t\t\tFormat[\n\t\t\t\tx_Real, \n\t\t\t\tInputForm\
\n\t\t\t\t] := \n\t\t\t\tOutputForm[\n\t\t\t\t\tNumberForm[\n\t\t\t\t\t\tx, \n\
\t\t\t\t\t\t5, \n\t\t\t\t\t\tNumberFormat \[Rule] \((If[#3 \[Equal] "\<\>", \
#1, SequenceForm[#1, "\<*^\>", #3]] &)\)\n\t\t\t\t\t\t]\n\t\t\t\t\t]; 
              Protect @@ 
                ps; \n\t\t\t\tWriteString[\n\t\t\t\t\tfilename, \n\t\t\t\t\t\
ToString[Live3DForm[graphics, animOpts], 
                  CharacterEncoding \[Rule] 
                    None]\n\t\t\t\t\t]; \n\t\t\t\tClose[
                filename]; \n\t\t\t\tps = 
                Unprotect[Real]; \n\t\t\t\tFormat[x_Real, 
                  InputForm] =. ; \n\t\t\t\tProtect @@ 
                ps;\n\t\t\t];\)}]}]], "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Close Private Contex", "Section"],

Cell[BoxData[
    \( (*\nEnd[]; \nProtect[];\n*) \)], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Close Package Contex", "Section"],

Cell[BoxData[
    \( (*\nEndPackage[]\n*) \)], "InputOnly"]
}, Closed]]
}, Closed]],

Cell["End Package", "Subtitle"],

Cell[CellGroupData[{

Cell["Applications of Self-Organizing Maps", "Title"],

Cell[CellGroupData[{

Cell[TextData[{
  "Setup the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " environment"
}], "Section"],

Cell["\<\
Global commands to obtain better plots and avoid some warning.\
\>", "Text"],

Cell[BoxData[{
    \(\($TextStyle = {FontFamily -> "\<Helvetica\>"};\)\), "\n", 
    \(\(Off[CompiledFunction::"\<cfsa\>"];\)\), "\n", 
    \(\(Off[Graphics3D::"\<gprim\>"];\)\)}], "InputOnly"]
}, Closed]],

Cell[CellGroupData[{

Cell["Set the geometry of input and output spaces", "Section"],

Cell[TextData[{
  "Here the geometric dimension of the input space (that is, the number of \
components of the input space), and the structure of the output layer (that \
is, the number of rows, columns, and planes of neurons) is selected. If a ",
  StyleBox["dimInputSpace",
    FontFamily->"Courier New"],
  " value greater than 3 is assigned, the algorithm executes normally but the \
plot functions are (obviously) limite to the first three components of the \
input space. The structure of the input range list must reflect the value \
given to ",
  StyleBox["dimInputSpace",
    FontFamily->"Courier New"],
  " (that is, if  ",
  StyleBox["dimInputSpace",
    FontFamily->"Courier New"],
  "=1 it must be have the form ",
  StyleBox["{{a1,b1}} ",
    FontFamily->"Courier New"],
  "if  ",
  StyleBox["dimInputSpace",
    FontFamily->"Courier New"],
  "=2 it must be have the form ",
  StyleBox["{{a1,b1},{a2,b2}}",
    FontFamily->"Courier New"],
  ", and so on. This is automatically taken care of if the input range is \
generated with the default statement provide below. However, it is obviously \
possible to defines ranges manually, using statements such as ",
  StyleBox["dimInputSpace={{0,1},{0,3}} ",
    FontFamily->"Courier New"],
  "and",
  StyleBox[" dimInputSpace={{0,1},{0,3},{1,2}}.",
    FontFamily->"Courier New"]
}], "Text"],

Cell[BoxData[{
    \(\(\(setGeometricParameters[\n\tdimInputSpace \[Rule] 2, \n\t
        numRowsOutputSpace \[Rule] 5, \n\t
        numColumnsOutputSpace \[Rule] 5, \n\t
        numPlanesOutputSpace \[Rule] 1\n\t];\)\(\n\)
    \)\t\), "\n", 
    \(\(inputRange = 
        Table[{0, 1}, {i, 1, dimInputSpace[]}];\)\)}], "InputOnly",
  Background->RGBColor[1, 0, 1],
  CellTags->"geometry"],

Cell[BoxData[
    \("\<Input Range = \>" <> ToString[inputRange]\)], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Display and set the SOM algorithm functions and parameters", "Section"],

Cell[CellGroupData[{

Cell["Weight update formula", "Subsubsection"],

Cell[BoxData[
    \(\(\*"\"\<\!\(w\_p\)(n+1)=\!\(w\_p\)(n)+\[Eta](n)\!\(h\_\(p, p\_winner\)\
\)(x(n)-\!\(w\_p\)(n))\>\"";\)\)], "InputOnly"],

Cell["Winner selection criterion", "Subsubsection"],

Cell[TextData[{
  "Here the matching function, that is, the function used to select the \
winning output neuron, is set at the default value, which is also displayed. \
The matching function can be assigned manually passing an optional parameter \
to the function. See function usage with ",
  StyleBox["?functionName.",
    FontFamily->"Courier New"]
}], "Text"],

Cell[BoxData[{
    \(\(setMatchFunction[];\)\), "\n", 
    \(\*"\"\<\!\(p\_winner\)=p:\!\(min\+p\)\>\"" \(matchFunction[]\)[x, 
        w\_p]\)}], "Input"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Neighborhood function", "Subsubsection"],

Cell[TextData[{
  "Here the distance and neighborhood functions, that is, the function used \
within the neighborhood to determine the distance between output neurons, and \
the function used within the algorithm as a multiplicative coefficient of the \
weight update term, are set at the default value, which is also displayed.  \
Both  functions can be assigned manually passing an optional parameter to the \
function.  See function usage with ",
  StyleBox["?functionName.",
    FontFamily->"Courier New"]
}], "Text"],

Cell[BoxData[
    \(\(\(\n\)\(\(setDistanceFunction[distanceFunction \[Rule] d];\)\n
    \(setNeighborhoodFunction[sigma0 \[Rule] \[Sigma]\_o, 
        tau1 -> \[Tau]\_1];\)\n
    {\*"\"\<\!\(h\_\(p, q\)\)(n)=\>\"" \(neighborhoodFunction[]\)[ip, jp, kp, 
          iq, jq, kq, n], \n\*"\"\<\!\(\[Sigma]\_0\)=\>\"" <> 
        ToString[
          N[sigma0 /. Options[setNeighborhoodFunction]]]}\)\)\)], "Input"],

Cell[BoxData[{
    \(\(setDistanceFunction[];\)\), "\n", 
    \("\<d[ip,jp,kp,iq,jq,kq]=\>" \(distanceFunction[]\)[ip, jp, kp, iq, jq, 
        kq]\)}], "Input"],

Cell[TextData[{
  "The value of the relaxation parameter ",
  StyleBox["tau1",
    FontFamily->"Courier New"],
  " of the default neighborhood function is set. Change this value and \
observe on the plots  below the effect of this parameter on the shape of the \
neighborhood function, as the algorithm's step counter increases!"
}], "Text"],

Cell[BoxData[
    \(\(setNeighborhoodFunction[tau1 \[Rule] 1000];\)\)], "InputOnly",
  Background->RGBColor[1, 0, 1]],

Cell[BoxData[
    \(\(\(Table[\n\t
        Plot3D[\n\t\t\(neighborhoodFunction[]\)[0, 0, 0, x, y, 0, 
            n], \n\t\tEvaluate[{x, \(-\((numRowsOutputSpace[]*1.01 - 1)\)\)/
                2, \((numRowsOutputSpace[]*1.01 - 1)\)/
                2}], \n\t\tEvaluate[{y, \(-\((numColumnsOutputSpace[]*1.01 - 
                      1)\)\)/2, \((numColumnsOutputSpace[]*1.01 - 1)\)/
                2}], \n\t\tPlotRange \[Rule] {All, 
              All, {0, 1}}, \n\t\tPlotPoints \[Rule] 
            50, \n\t\tPlotLabel \[Rule] \*"\"\<Neighborhood function \
\!\(h\_\(p, q\)\)(n), n = \>\"" <> ToString[n], \n\t\tAmbientLight \[Rule] 
            RGBColor[0, 1, 0], \n\t\tBoxRatios \[Rule] 
            Automatic, \n\t\tTicks \[Rule] {Automatic, 
              Automatic, {0, 0.5, 1}}\n\t\t], \n\t{n, 0, 2000, 
          500}\n\t];\)\(\n\)
    \)\)], "Input",
  CellOpen->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Learning Rate", "Subsubsection"],

Cell[TextData[{
  "Here the learning rate function, that is, the function used within the \
algorithm as a (usually decreasing with time) multiplicative coefficient of \
the weight update term, is set at the default value, which is also displayed. \
The learning rate  function can be assigned manually passing an optional \
parameter to the function. See function usage with ",
  StyleBox["?functionName.",
    FontFamily->"Courier New"]
}], "Text"],

Cell[BoxData[{
    \(\(setLearningRateFunction[eta0 \[Rule] \[Eta]\_o, 
        tau2 \[Rule] \[Tau]\_2];\)\), "\n", 
    \("\<\[Eta](n)=\>" \(learningRateFunction[]\)[n]\)}], "Input"],

Cell[TextData[{
  "The value of the multiplicative constant ",
  StyleBox["eta0",
    FontFamily->"Courier New"],
  " and of the relaxation parameter ",
  StyleBox["tau2",
    FontFamily->"Courier New"],
  " of the default learning rate function is set. Change these values and \
observe on the plots  below the effect of this parameter on the shape of the \
learning rate function, and of the whole multiplicative term composed by the \
learning rate function and the neighborhood function, as the algorithm's step \
counter increases!"
}], "Text"],

Cell[BoxData[
    \(\(setLearningRateFunction[eta0 \[Rule] 0.1, 
        tau2 \[Rule] 1000];\)\)], "InputOnly",
  Background->RGBColor[1, 0, 1]],

Cell[BoxData[
    \(\(Table[\n\t
        Show[\n\tGraphicsArray[\n\t{\n\t
              Plot3D[\n\t\t\(neighborhoodFunction[]\)[0, 0, 0, x, y, 0, 
                  n], \n\t\tEvaluate[{x, \(-numRowsOutputSpace[]\)/2, 
                    numRowsOutputSpace[]/
                      2}], \n\t\tEvaluate[{y, \(-numColumnsOutputSpace[]\)/2, 
                    numColumnsOutputSpace[]/
                      2}], \n\t\tPlotRange \[Rule] {All, 
                    All, {0, 1}}, \n\t\tPlotPoints \[Rule] 
                  50, \n\t\tPlotLabel \[Rule] \*"\"\<\!\(h\_\(p, q\)\)(n), n \
= \>\"" <> ToString[n], \n\t\tAmbientLight \[Rule] 
                  RGBColor[0, 1, 0], \n\t\tTicks \[Rule] {None, None, 
                    Automatic}, \n\t\tViewPoint -> {1.022, \ \(-3.203\), \ 
                    0.379}, \n\t\tDisplayFunction \[Rule] 
                  Identity\n\t\t], \n\t
              Plot3D[\n\t\t\(learningRateFunction[]\)[
                    n]*\n\t\t\(neighborhoodFunction[]\)[0, 0, 0, x, y, 0, 
                    n], \n\t\tEvaluate[{x, \(-numRowsOutputSpace[]\)/2, 
                    numRowsOutputSpace[]/
                      2}], \n\t\tEvaluate[{y, \(-numColumnsOutputSpace[]\)/2, 
                    numColumnsOutputSpace[]/
                      2}], \n\t\tPlotRange \[Rule] {All, 
                    All, {0, globalEta0}}, \n\t\tTicks \[Rule] {None, None, 
                    Automatic}, \n\t\tPlotPoints \[Rule] 
                  30, \n\t\tPlotLabel \[Rule] \*"\"\<\[Eta](n)*\!\(h\_\(p, \
q\)\)(n), n = \>\"" <> ToString[n], \n\t\tAmbientLight \[Rule] 
                  RGBColor[1, 0, 0], \n\t\tTicks \[Rule] 
                  None, \n\t\tViewPoint -> {1.022, \ \(-3.203\), \ 
                    0.379}, \n\t\tDisplayFunction \[Rule] 
                  Identity\n\t\t]\n\t\t}\n\t\t], \t\t\n\t\tDisplayFunction \
\[Rule] $DisplayFunction, \n\t\tImageSize \[Rule] 500\n\t\t], \t\t\n\t{n, 0, 
          2500, 500}\n\t];\)\)], "Input",
  CellOpen->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Initialize the algorithm", "Section"],

Cell[BoxData[
    \(\(initializeAlgorithm[];\)\)], "InputOnly"],

Cell[CellGroupData[{

Cell["Generate initial weights", "Subsubsection"],

Cell["\<\
To enhance the visual impact of the initial ordering phase, the initial \
weights are assigned with random uniform values in the whole input range. It \
is however more common to select in this case a small subset of the input \
range.\
\>", "Text"],

Cell[BoxData[
    \(\(weightList = \n\t
        generateUniformlyDistributedRandomWeightList[\n\t\tinputRange\n\t\t];\
\)\)], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Define the training set", "Section"],

Cell[CellGroupData[{

Cell[TextData[{
  "Define the following training set generating procedures:\n",
  StyleBox["\tuniformRandomTrainingPointList\n\tblockRandomTrainingPointList\n\
\tinterspersedBlockRandomTrainingPointList\n\t\
hollowEllipsoidRandomTrainingPointList", "Commentary",
    FontFamily->"Courier New",
    FontSize->12,
    FontWeight->"Bold"]
}], "Subsubsection"],

Cell["\<\
Here you can find some examples of procedures generating lists of random \
training points.\
\>", "Text"],

Cell[BoxData[{
    \(\(Clear[uniformRandomTrainingPointList];\)\), "\n", 
    \(\(uniformRandomTrainingPointList[\n\tnumberOfTrainingPoints_, \n\t
          inputRange_\n\t] := \n\t
        generateUniformlyDistributedRandomTrainingPointList[\n\t\t\
numberOfTrainingPoints, \n\t\tinputRange\n\t\t];\)\)}], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]],

Cell[BoxData[{
    \(\(Clear[blockRandomTrainingPointList];\)\), "\n", 
    \(\(blockRandomTrainingPointList[\n\tnumberOfTrainingPoints_, \n\t
          inputRange_\n\t] := \n\t
        Join[\n\t\tgenerateUniformlyDistributedRandomTrainingPointList[\n\t\t\
\tCeiling[numberOfTrainingPoints/2], \n\t\t\tinputRange/
              2\n\t\t\t], \t\n\t\t\
generateUniformlyDistributedRandomTrainingPointList[\n\t\t\tCeiling[
              numberOfTrainingPoints/2], \n\t\t\tinputRange/2 + 
              Apply[Plus, inputRange, {1}]/
                2\n\t\t\t]\n\t\t];\)\)}], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]],

Cell[BoxData[{
    \(\(Clear[interspersedBlockRandomTrainingPointList];\)\), "\n", 
    \(\(interspersedBlockRandomTrainingPointList[\n\t
          numberOfTrainingPoints_, \n\tinputRange_\n\t] := \n\t
        Module[\n\t\t{\n\t\ttpl\n\t\t}, \n\t\ttpl = 
            Flatten[Table[\n\t\t\tIf[
                  Random[] > 
                    0.5, \n\t\t\t\t\
generateUniformlyDistributedRandomTrainingPointList[\n\t\t\t\t\t1, \n\t\t\t\t\
\tinputRange/
                      2\n\t\t\t\t\t], \t\n\t\t\t\t\
generateUniformlyDistributedRandomTrainingPointList[\n\t\t\t\t\t1, \n\t\t\t\t\
\tinputRange/2 + Apply[Plus, inputRange, {1}]/
                        2\n\t\t\t\t\t]\n\t\t\t\t], \n\t\t\t\t{i, 1, 
                  numberOfTrainingPoints}\n\t\t], 
              1]\n\t\t];\)\)}], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]],

Cell[BoxData[{
    \(\(Clear[hollowEllipsoidRandomTrainingPointList];\)\), "\n", 
    \(\(hollowEllipsoidRandomTrainingPointList[\n\t
          numberOfTrainingPoints_, \n\t{\n\txRange_List, \n\t
            yRange_List: {0, 0}, \n\tzRange_List: {0, 0}\n\t}, \n\t
          thicknessRatio_:  4\n\t] := \n\t
        Module[\n\t\t{\n\t\ttheta, \n\t\tphi, \n\t\tinputRange\n\t\t}, \n\t\t\
inputRange = {xRange, yRange, 
              zRange}; \n\t\tTable[\n\t\t\tApply[Plus, inputRange, {1}]/
                2 + \n\t\t\tMap[
                  Random[Real, #] &, \n\t\t\t\tAbs[
                    Thread[List[\n\t\t\t\t\tApply[#2 - #1 &, inputRange, {1}]/
                            2*\((thicknessRatio - 1)\)/
                            thicknessRatio, \n\t\t\t\t\tApply[#2 - #1 &, 
                            inputRange, {1}]/
                          2\n\t\t\t\t\t]]]\n\t\t\t\t]*\n\t\t\tIf[
                  dimInputSpace[] == 
                    3, \n\t\t\t\t{\n\t\t\t\tCos[
                        theta = Random[Real, {0, 2  Pi}]]\n\t\t\t\tCos[
                        phi = Random[Real, {\(-Pi\)/2, Pi/2}]], \n\t\t\t\tCos[
                        theta] Sin[phi], \n\t\t\t\tSin[
                      theta]\n\t\t\t\t}, \n\t\t\t\t{\n\t\t\t\tN[
                      Cos[theta = Random[Real, {\(-Pi\), Pi}]]], \n\t\t\t\tN[
                      Sin[theta]], \n\t\t\t\t0.0\n\t\t\t\t}\n\t\t\t\t], \n\t\t\
\t{i, 1, numberOfTrainingPoints}\n\t\t\t]\n\t\t];\)\)}], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]]
}, Closed]],

Cell[CellGroupData[{

Cell["Generate the training set", "Subsubsection"],

Cell[TextData[{
  "Using the procedures given above, the set of training points is generated. \
A given set can be utilized more than once, cyclically, setting for ",
  StyleBox["numberOfRepetitions ",
    FontFamily->"Courier New"],
  "a value greater than 1."
}], "Text"],

Cell[BoxData[{\(numberOfTrainingPoints = \ 1000;\), "\n", 
    RowBox[{\(numberOfRepetitions = \ 5;\), "\n"}], "\n", 
    RowBox[{
      RowBox[{"trainingPointList", "=", 
        RowBox[{
          StyleBox["uniformRandomTrainingPointList",
            "Commentary",
            FontFamily->"Courier New",
            FontSize->12,
            FontWeight->"Bold"], "[", "\n", 
          "\t", \(numberOfTrainingPoints, \n\tinputRange\), "\n", "\t", 
          "]"}]}], ";"}]}], "InputOnly",
  Background->RGBColor[1, 0, 1],
  CellTags->"training"],

Cell[BoxData[
    \(\(trainingPointList = 
        Join[Flatten[Table[trainingPointList, {i, 1, numberOfRepetitions}], 
            1]];\)\)], "InputOnly"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Partition the training set ",
  StyleBox["(to avoid generating unnecessary intermediate data sets)", 
    "Commentary"]
}], "Subsubsection"],

Cell[TextData[{
  "The list of training points is partitioned according to the criterion \
given by a list of separators. This allows the generation of a limited number \
of intermediate results during the execution of the algorithm, with an \
emphasis on the initial phase of the unfolding process. In this way one can \
avoid saving and plotting thousands of almost indistinguishable \
configurations. The statement ",
  StyleBox["separatorList={1,Length[trainingPointList]}; ",
    FontFamily->"Courier New",
    FontWeight->"Bold"],
  "leads to the visualization of the first and last setp only. Comment it \
(with ",
  StyleBox["(*...*) ",
    FontFamily->"Courier New",
    FontWeight->"Bold"],
  ") to display a selection of the intermediate steps."
}], "Text"],

Cell[BoxData[{
    \(\("\<separators = \>"\  <> 
        ToString[\n\t
          separatorList = \n\t\tgenerateSeparatorList[\n\t\t\tLength[
                trainingPointList]\n\t\t\t]\n\t\t];\)\n\t\t\n \
(*\(separatorList = {1, Length[trainingPointList]};\)*) \n\t\), "\n", 
    \(\(partitionedTrainingPointList = \n\t
        partitionList[\n\t\ttrainingPointList, \n\t\tseparatorList\n\t\t\
];\)\)}], "InputOnly",
  Background->RGBColor[0.773449, 0.773449, 0.996109]],

Cell[BoxData[
    \("\<Plot shown at steps: \>" <> ToString[separatorList]\)], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Show the training set", "Subsubsection"],

Cell["\<\
The list of training points is plotted. Select a different viewpoint pasting \
from the menu Input->3D Viewpoint Selector.\
\>", "Text"],

Cell[BoxData[
    \(\(trainingPointsPlot = 
        showTrainingPoints[\n\ttrainingPointList, \n\t
          trainingPointSize \[Rule] 0.015, \n\tPlotRange -> inputRange, \n\t
          PlotLabel -> "\<Distribution of training points\>", \n\t
          ViewPoint -> {1.300, \ \(-2.400\), \ 2.000}\n\t];\)\)], "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Run the algorithm and display the results", "Section"],

Cell["\<\
The algorithm is executed with a given initial set of weights, and a given \
sequence of training points. The result is a sequence of weight sets.\
\>", "Text"],

Cell[BoxData[
    \("\<Execution time = \>" <> 
      ToString[First[\n\t
          Timing[\n\t\t\(historyOfWeights = \n\t\t\t\
sequenceOfUpdatesWeightList[\n\t\t\tweightList, \n\t\t\t\
partitionedTrainingPointList\n\t\t\t];\)\n\t\t]\n\t]]\)], "Input"],

Cell["\<\
The  sequence of weight sets produced by the algorithm is plotted. Select a \
different viewpoint pasting from the menu Input->3D Viewpoint Selector. You \
can switch on and off the visualization of the lines joining the nodes of the \
output layer, of the winning node of each step (which appears as a \
star-shaped polyhedron), of the training point used in each step  (which \
appears as a dark dodecahedron), and of the whole set of training points.\
\>", "Text"],

Cell[BoxData[
    \(\(historyPlot = 
        showHistory[\n\thistoryOfWeights, \n\tseparatorList, \n\t
          trainingPointList, \n\tcuboidSideLength \[Rule] 0.035, \n\t
          showLines \[Rule] True, \n\tlineThickness \[Rule] 0.001, \n\t
          showWinner \[Rule] True, \n\ttrainingPointSize \[Rule] 0.015, \n\t
          showActiveTrainingPoint \[Rule] True, \n\t
          showAllTrainingPoints \[Rule] False, \n\t
          showOnlyFinalConfiguration \[Rule] False, \n\t
          PlotRange -> inputRange, \n\t
          ViewPoint -> {1.300, \ \(-2.400\), \ 2.000}\n\t];\)\)], "Input",
  AnimationDisplayTime->0.107276],

Cell[BoxData[{
    RowBox[{
      ButtonBox["geometry",
        ButtonData:>"geometry",
        ButtonStyle->"Hyperlink"], ";"}], "\n", 
    RowBox[{
      ButtonBox["training",
        ButtonData:>"training",
        ButtonStyle->"Hyperlink"], ";"}]}], "InputOnly"],

Cell[BoxData[
    \(\(Show[{trainingPointsPlot, Last[historyPlot]}];\)\)], "Input"],

Cell[CellGroupData[{

Cell["Generate LiveGraphics3D animation file", "Subsubsection"],

Cell[BoxData[{
    \(\(SetDirectory["\<D:\Documenti\EastDocs\SitoWebClaudio\education\G\
allery\>"];\)\), "\n", 
    \(\(WriteLive3DForm[\n\t"\<test.m\>", \n\thistoryPlot, \n\t
        AnimationDisplayTime \[Rule] 0.5, \n\t
        AnimationDirection \[Rule] Forward\n\t];\)\)}], "InputOnly",
  Evaluatable->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["References", "Section"],

Cell[TextData[{
  "Haykin,  S. (1999). ",
  StyleBox["Neural Networks: A Comprehensive Foundation",
    FontSlant->"Italic"],
  ", 2nd ed., Prentice Hall, Upper Saddle River, NJ.\nKohonen, T. (2001). ",
  StyleBox["Self-Organizing Maps",
    FontSlant->"Italic"],
  ", 3rd ed., Springer-Verlag, Berlin.\nSpitzer, M. (1999). ",
  StyleBox["The Mind Within the Net",
    FontSlant->"Italic"],
  ", The MIT Press, Cambridge, MA.\n"
}], "Text"]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1600}, {0, 1147}},
AutoGeneratedPackage->Automatic,
PrintingStyleEnvironment->"Printout",
CellGrouping->Manual,
WindowSize->{1013, 677},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingPageRange->{Automatic, Automatic},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {36, 36}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->True,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->True},
ShowCellLabel->False,
StyleDefinitions -> "DemoText.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "geometry"->{
    Cell[39008, 888, 389, 9, 188, "InputOnly",
      CellTags->"geometry"]},
  "training"->{
    Cell[51363, 1194, 548, 13, 168, "InputOnly",
      CellTags->"training"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"geometry", 58102, 1385},
  {"training", 58195, 1388}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 45, 0, 70, "Title"],
Cell[1802, 53, 137, 5, 122, "Text"],

Cell[CellGroupData[{
Cell[1964, 62, 25, 0, 41, "Subtitle"],
Cell[1992, 64, 1251, 25, 708, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[3280, 94, 37, 0, 45, "Subtitle"],

Cell[CellGroupData[{
Cell[3342, 98, 24, 0, 54, "Section"],
Cell[3369, 100, 109, 2, 88, "InputOnly"],
Cell[3481, 104, 200, 3, 68, "InputOnly"],
Cell[3684, 109, 212, 3, 68, "InputOnly"],
Cell[3899, 114, 226, 4, 88, "InputOnly"],
Cell[4128, 120, 220, 3, 88, "InputOnly"],
Cell[4351, 125, 492, 7, 128, "InputOnly"],
Cell[4846, 134, 336, 5, 88, "InputOnly"],
Cell[5185, 141, 288, 4, 88, "InputOnly"],
Cell[5476, 147, 241, 3, 88, "InputOnly"],
Cell[5720, 152, 400, 6, 108, "InputOnly"],
Cell[6123, 160, 620, 8, 148, "InputOnly"],
Cell[6746, 170, 341, 5, 88, "InputOnly"],
Cell[7090, 177, 602, 8, 148, "InputOnly"],
Cell[7695, 187, 502, 7, 128, "InputOnly"],
Cell[8200, 196, 741, 10, 188, "InputOnly"],
Cell[8944, 208, 452, 7, 128, "InputOnly"],
Cell[9399, 217, 341, 5, 108, "InputOnly"],
Cell[9743, 224, 193, 3, 68, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[9973, 232, 26, 0, 34, "Section"],
Cell[10002, 234, 307, 6, 168, "InputOnly"],
Cell[10312, 242, 295, 5, 228, "InputOnly"],
Cell[10610, 249, 337, 6, 188, "InputOnly"],
Cell[10950, 257, 690, 11, 328, "InputOnly"],
Cell[11643, 270, 178, 3, 128, "InputOnly"],
Cell[11824, 275, 783, 14, 348, "InputOnly"],
Cell[12610, 291, 250, 4, 148, "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12909, 301, 41, 0, 45, "Subtitle"],

Cell[CellGroupData[{
Cell[12975, 305, 39, 0, 54, "Section"],
Cell[13017, 307, 85, 1, 108, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[13139, 313, 41, 0, 34, "Section"],
Cell[13183, 315, 1509, 31, 708, "InputOnly"],
Cell[14695, 348, 234, 4, 108, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[14966, 357, 41, 0, 34, "Section"],
Cell[15010, 359, 546, 10, 308, "InputOnly"],
Cell[15559, 371, 90, 1, 48, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[15686, 377, 36, 0, 34, "Section"],
Cell[15725, 379, 300, 4, 228, "InputOnly"],
Cell[16028, 385, 82, 1, 48, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[16147, 391, 40, 0, 34, "Section"],
Cell[16190, 393, 453, 7, 288, "InputOnly"],
Cell[16646, 402, 88, 1, 48, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[16771, 408, 33, 0, 34, "Section"],
Cell[16807, 410, 340, 5, 248, "InputOnly"],
Cell[17150, 417, 79, 1, 48, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[17266, 423, 43, 0, 34, "Section"],
Cell[17312, 425, 208, 3, 188, "InputOnly"],
Cell[17523, 430, 899, 14, 608, "InputOnly"],
Cell[18425, 446, 927, 17, 608, "InputOnly"],
Cell[19355, 465, 697, 11, 468, "InputOnly"],
Cell[20055, 478, 696, 12, 388, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[20788, 495, 35, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[20848, 499, 36, 0, 44, "Subsubsection"],
Cell[20887, 501, 1471, 27, 368, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[22395, 533, 39, 0, 28, "Subsubsection"],
Cell[22437, 535, 544, 9, 228, "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[23030, 550, 38, 0, 34, "Section"],
Cell[23071, 552, 74, 1, 48, "InputOnly"],

Cell[CellGroupData[{
Cell[23170, 557, 33, 0, 44, "Subsubsection"],
Cell[23206, 559, 2269, 41, 828, "InputOnly"],
Cell[25478, 602, 4566, 78, 1528, "InputOnly"],
Cell[30047, 682, 3159, 53, 1328, "InputOnly"],
Cell[33209, 737, 811, 12, 528, "InputOnly"],
Cell[34023, 751, 495, 7, 308, "InputOnly"]
}, Open  ]],

Cell[CellGroupData[{
Cell[34555, 763, 39, 0, 44, "Subsubsection"],
Cell[34597, 765, 2101, 39, 1248, "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[36747, 810, 39, 0, 34, "Section"],
Cell[36789, 812, 66, 1, 108, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[36892, 818, 39, 0, 34, "Section"],
Cell[36934, 820, 59, 1, 88, "InputOnly"]
}, Closed]]
}, Closed]],
Cell[37020, 825, 31, 0, 45, "Subtitle"],

Cell[CellGroupData[{
Cell[37076, 829, 53, 0, 70, "Title"],

Cell[CellGroupData[{
Cell[37154, 833, 115, 5, 54, "Section"],
Cell[37272, 840, 86, 2, 29, "Text"],
Cell[37361, 844, 193, 3, 88, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[37591, 852, 62, 0, 34, "Section"],
Cell[37656, 854, 1349, 32, 124, "Text"],
Cell[39008, 888, 389, 9, 188, "InputOnly",
  CellTags->"geometry"],
Cell[39400, 899, 77, 1, 40, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39514, 905, 77, 0, 54, "Section"],

Cell[CellGroupData[{
Cell[39616, 909, 46, 0, 44, "Subsubsection"],
Cell[39665, 911, 140, 2, 49, "InputOnly"],
Cell[39808, 915, 51, 0, 44, "Subsubsection"],
Cell[39862, 917, 363, 7, 48, "Text"],
Cell[40228, 926, 155, 3, 69, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40420, 934, 46, 0, 44, "Subsubsection"],
Cell[40469, 936, 521, 9, 67, "Text"],
Cell[40993, 947, 410, 7, 120, "Input"],
Cell[41406, 956, 161, 3, 60, "Input"],
Cell[41570, 961, 341, 7, 48, "Text"],
Cell[41914, 970, 117, 2, 48, "InputOnly"],
Cell[42034, 974, 881, 16, 19, "Input",
  CellOpen->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[42952, 995, 38, 0, 44, "Subsubsection"],
Cell[42993, 997, 450, 8, 67, "Text"],
Cell[43446, 1007, 183, 3, 60, "Input"],
Cell[43632, 1012, 549, 12, 67, "Text"],
Cell[44184, 1026, 144, 3, 48, "InputOnly"],
Cell[44331, 1031, 1982, 33, 19, "Input",
  CellOpen->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[46362, 1070, 43, 0, 54, "Section"],
Cell[46408, 1072, 63, 1, 48, "InputOnly"],

Cell[CellGroupData[{
Cell[46496, 1077, 49, 0, 44, "Subsubsection"],
Cell[46548, 1079, 259, 5, 48, "Text"],
Cell[46810, 1086, 192, 4, 108, "InputOnly"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[47051, 1096, 42, 0, 34, "Section"],

Cell[CellGroupData[{
Cell[47118, 1100, 356, 8, 156, "Subsubsection"],
Cell[47477, 1110, 115, 3, 29, "Text"],
Cell[47595, 1115, 372, 6, 208, "InputOnly"],
Cell[47970, 1123, 636, 11, 328, "InputOnly"],
Cell[48609, 1136, 849, 16, 488, "InputOnly"],
Cell[49461, 1154, 1536, 25, 848, "InputOnly"]
}, Closed]],

Cell[CellGroupData[{
Cell[51034, 1184, 50, 0, 28, "Subsubsection"],
Cell[51087, 1186, 273, 6, 48, "Text"],
Cell[51363, 1194, 548, 13, 168, "InputOnly",
  CellTags->"training"],
Cell[51914, 1209, 155, 3, 48, "InputOnly"]
}, Open  ]],

Cell[CellGroupData[{
Cell[52106, 1217, 159, 4, 44, "Subsubsection"],
Cell[52268, 1223, 768, 16, 88, "Text"],
Cell[53039, 1241, 470, 9, 308, "InputOnly"],
Cell[53512, 1252, 87, 1, 40, "Input"]
}, Closed]],

Cell[CellGroupData[{
Cell[53636, 1258, 46, 0, 28, "Subsubsection"],
Cell[53685, 1260, 146, 3, 29, "Text"],
Cell[53834, 1265, 317, 5, 160, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[54200, 1276, 60, 0, 54, "Section"],
Cell[54263, 1278, 170, 3, 29, "Text"],
Cell[54436, 1283, 252, 5, 200, "Input"],
Cell[54691, 1290, 477, 7, 67, "Text"],
Cell[55171, 1299, 632, 11, 320, "Input"],
Cell[55806, 1312, 266, 8, 73, "InputOnly"],
Cell[56075, 1322, 83, 1, 40, "Input"],

Cell[CellGroupData[{
Cell[56183, 1327, 63, 0, 44, "Subsubsection"],
Cell[56249, 1329, 314, 6, 168, "InputOnly",
  Evaluatable->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[56624, 1342, 29, 0, 54, "Section"],
Cell[56656, 1344, 440, 11, 122, "Text"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

